﻿--[[---------------------------------------------------------------------------------
 ADDON DECLARATION && SAVED VARIABLES && MENU INITIALIZATION
------------------------------------------------------------------------------------]]
SacredBuff = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0")
local L = AceLibrary("AceLocale-2.2"):new("SacredBuff")
local PT = AceLibrary("PeriodicTable-2.0")
local BS = AceLibrary("Babble-Spell-2.2")
local M = AceLibrary("Metrognome-2.0")
local G = AceLibrary("Gratuity-2.0")
local _G = getfenv(0)
local _, _, rev = string.find("$Rev: 19848 $", "(%d+)")
SacredBuff.fileRevisions = { ["SacredBuff.lua"] = tonumber(rev), ["UI.lua"] = 0 }
rev = nil

-- BINDINGS
-----------------------------------------------------------
BINDING_HEADER_SACREDBUFF = L["SacredBuff"]
BINDING_NAME_MOUNT = L["Mount"]

-- SAVED VARIABLES
-----------------------------------------------------------
SacredBuff:RegisterDB("SBDB", "SBPCDB")

SacredBuff:RegisterDefaults("profile", {
	movable = true,
	tooltip = 2,
	toggle = true,
	scaleSize = 1
})

SacredBuff:RegisterDefaults("char", {
	groupMode = 0,
	sacredCount = 0,
	holyCount = 0,
	
	["mounts60"] = {},
	["mounts100"] = {},
	["mountsaq100"] = {},
	
	spells = {
		[ BS["Inner Fire"] ] = false,
		[ BS["Resurrection"] ] = false, 
		
		[ BS["Divine Spirit"] ] = false,
		[ BS["Power Word: Fortitude"] ] = false,
		[ BS["Shadow Protection"] ] = false,
		
		["TALENT"] = false,
		["RACIAL"] = false,
		
		[ BS["Prayer of Spirit"] ] = false,
		[ BS["Prayer of Fortitude"] ] = false,
		[ BS["Prayer of Shadow Protection"] ] = false
	}
})

SacredBuff.talents = {
	BS["Pain Suppression"],
	BS["Circle of Healing"],
	BS["Vampiric Touch"],
	BS["Power Infusion"],
	BS["Lightwell"],
	BS["Shadowform"],
	BS["Inner Focus"]
}

SacredBuff.racials = {
	BS["Touch of Weakness"],
	BS["Shadowguard"],
	BS["Fear Ward"],
	BS["Elune's Grace"],
	BS["Feedback"]
}

SacredBuff.attributeQueue = {}

-- MENU INITIALIZATION
-----------------------------------------------------------
SacredBuff:RegisterChatCommand(L["Chat_Commands"], {
	type = "group",
	args = {
		[ L["Lock"] ] = {
			name = L["Lock"], type = "toggle",
			desc = L["(Un)Lock the SacredBuff frame."],
			map = { [false] = "|cffff0000"..L["On"].."|r", [true] = "|cff00ff00"..L["Off"].."|r" },
			get = function()
				return SacredBuff.db.profile.movable
			end,
			set = function()
				SacredBuff.db.profile.movable = not SacredBuff.db.profile.movable
				if ( SacredBuff.db.profile.movable ) then
					SacredBuff.frame.title:Show()
					SacredBuff.frame:SetBackdropColor(0.00, 0.00, 0.00, 0.60)
				else
					SacredBuff.frame.title:Hide()
					SacredBuff.frame:SetBackdropColor(0.00, 0.00, 0.00, 0.00)
				end
			end
		},
		[ L["Scale"] ] = {
			name = L["Scale"], type = "range",
			desc = L["Set the frame's scale."],
			min = 0.50, max = 2.00, isPercent = true,
			get = function() return SacredBuff.db.profile.scaleSize end,
			set = function(v)
				SacredBuff.db.profile.scaleSize = v
				SacredBuff:ScaleFrame()
			end,
		},
		[ L["Tooltip"] ] = {
			name = L["Tooltip Options"], type = "group",
			desc = L["Select what information the tooltip provides."],
			args = {
				[ L["None"] ] = {
					name = L["No Tooltip"], type = "execute",
					desc = L["No tooltip will be displayed over the spell icons."],
					func = function()
						if ( SacredBuff.db.profile.tooltip ~= 0 ) then
							SacredBuff.db.profile.tooltip = 0
							SacredBuff:Print(L["Tooltips turned off."])
						end
					end
				},
				[ MANA ] = {
					name = L["Mana Only"], type = "execute",
					desc = L["Only the mana cost of the spell will be displayed in its tooltip."],
					func = function()
						if ( SacredBuff.db.profile.tooltip ~= 1 ) then
							SacredBuff.db.profile.tooltip = 1
							SacredBuff:Print(L["Tooltips will now display the spell's mana cost."])
						end
					end
				},
				[ L["Full"] ] = {
					name = L["Full Tooltip"], type = "execute",
					desc = L["The spell's full tooltip will be displayed on mouseover."],
					func = function()
						if ( SacredBuff.db.profile.tooltip ~= 2 ) then
							SacredBuff.db.profile.tooltip = 2
							SacredBuff:Print(L["Full spell tooltips will now be displayed."])
						end
					end
				}
			}
		}
	}
})


--[[---------------------------------------------------------------------------------
 ADDON INITIALIZATION && ACTIVATION
------------------------------------------------------------------------------------]]
function SacredBuff:OnInitialize()
	local _, class = UnitClass("player")
	if ( class ~= "PRIEST" ) then
		self:Print(L["You are not a Priest."])
		self:ToggleActive(false)
		self:Print(L["This addon has been disabled."])
		return
	else
		self:ToggleActive(true)
	end
	if (( self.db.char.groupMode ) and ( type(self.db.char.groupMode) == "boolean" )) then
		self.db.char.groupMode = 0
	end
	self:CreateTooltip()
	self:CreateMainFrame()
end

function SacredBuff:OnEnable()
	if ( not self.db.char.playerLevel ) then
		self.db.char.playerLevel = UnitLevel("player")
	end
	
	if ( not self.db.char.ridingSkillIndex ) then
		self:SKILL_LINES_CHANGED()
	end

	self:RegisterBucketEvent("BAG_UPDATE", 1)
	
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("ZONE_CHANGED", "BAG_UPDATE")
	self:RegisterEvent("LEARNED_SPELL_IN_TAB")
	self:RegisterEvent("PLAYER_LEVEL_UP")
	self:RegisterEvent("SKILL_LINES_CHANGED")
	-- self:RegisterEvent("SPELL_UPDATE_COOLDOWN")
	-- self:RegisterEvent("SacredBuff_CooldownComplete")
	self:RegisterEvent("SacredBuff_ModeChanged")
	
	self:LEARNED_SPELL_IN_TAB()
	
	for _, s in ipairs(self.racials) do
		local r = self:FindSpell(s)
		if ( r ) then
			self.db.char.spells["RACIAL"] = r
			break
		end
	end
	
	self.frame:Show()
	
	self:ScheduleEvent(self.UpdateButtons, 0.5, self)
end

function SacredBuff:OnDisable()
	self.frame:Hide()
end


--[[---------------------------------------------------------------------------------
 EVENT HANDLERS
------------------------------------------------------------------------------------]]
function SacredBuff:BAG_UPDATE()
	self:UpdateCandleCount()
	self:UpdateMountLocs()
end

function SacredBuff:LEARNED_SPELL_IN_TAB()
	for s in next, self.db.char.spells do
		local tbl = self:FindSpell(s)
		if ( tbl ) then
			self.db.char.spells[s] = tbl
		else
			self.db.char.spells[s] = false
		end
	end
	for i = 1, #self.talents, 1 do
		local tbl = self:FindSpell(self.talents[i])
		if ( tbl ) then
			self.db.char.spells["TALENT"] = tbl
			return
		end
	end
end

function SacredBuff:PLAYER_LEVEL_UP(newLevel)
	self.db.char.playerLevel = newLevel
end

function SacredBuff:SKILL_LINES_CHANGED()
	for i = 1, GetNumSkillLines(), 1 do
		local skillName = GetSkillLineInfo(i)
		if ( skillName == L["Riding"] ) then
			local oldSkillIndex = self.db.char.ridingSkillIndex
			self.db.char.ridingSkillIndex = i
			if ( oldSkillIndex == 0 ) then
				self:UpdateMountLocs()
			end
			return
		end
	end
	self.db.char.ridingSkillIndex = 0
end
--[[
function SacredBuff:SPELL_UPDATE_COOLDOWN()
	for k in next, self.frame do
		if (( k ~= "main" ) and ( k ~= "mount" ) and ( k ~= "TALENT" ) and ( k ~= "RACIAL" )) then
			local texture = self.frame[k].icon
			local state = self.db.char.groupMode + 1
			local startTime, duration, enable = GetSpellCooldown(self.frame[k].states[state])
			if ( duration > 0 ) then
				SetDesaturation(texture, true)
				f.
			elseif ( duration == 0 ) then
				SetDesaturation(texture, false)
			end
			self.frame[k].icon:SetTexture(texture)
		end
	end
end

function SacredBuff:SacredBuff_CooldownComplete(frame)
	if ( self.frame[frame].isDesaturated ) then
		SetDesaturation(self.frame[frame].icon, false)
		self.frame[frame].isDesaturated = false
	end
end
--]]
function SacredBuff:PLAYER_REGEN_DISABLED()
	self.isCombating = true
end

function SacredBuff:PLAYER_REGEN_ENABLED()
	self.isCombating = false
	if ( ( self.attributeQueue ) and ( #self.attributeQueue > 0 ) ) then
		for i in ipairs(self.attributeQueue) do
			local tbl = table.remove(self.attributeQueue, i)
			_G[ tbl[1] ]:SetAttribute(tbl[2], tbl[3])
		end
	end
end

function SacredBuff:SacredBuff_ModeChanged()
	local state = self.db.char.groupMode
	for k in next, self.frame do
		if (( k == BS["Divine Spirit"] ) or ( k == BS["Power Word: Fortitude"] ) or ( k == BS["Shadow Protection"] )) then
			self.frame[k].icon:SetTexture( BS:GetSpellIcon( self.frame[k].states[ state + 1 ] ) )
		end
	end
end


--[[---------------------------------------------------------------------------------
 MOUNT-RELATED FUNCTIONS
------------------------------------------------------------------------------------]]
function SacredBuff:UpdateMountLocs()
	if (( self.db.char.playerLevel < 40 ) or ( self.db.char.ridingSkillIndex == 0 )) then
		return
	end
	local _, _, _, ridingSkill = GetSkillLineInfo(self.db.char.ridingSkillIndex)
	if ( ridingSkill >= 75 ) then
		self:IterateMounts("Mounts", 60)
		if ( ridingSkill >= 150 ) then
			self:IterateMounts("Mounts", 100)
			self:IterateMounts("Mounts - AQ", 100)
			self:UpdatePrefAQMount()	
		end
		self:UpdatePrefMount(ridingSkill)
	end
end

function SacredBuff:UpdatePrefMount(ridingSkill)
	if ( self.db.char.prefMount ) then
		local prefMount = self.db.char.prefMount
		if ( ridingSkill >= 150 ) then
			for i in ipairs(self.db.char["mounts100"]) do
				local name, bag, slot = strsplit("&", self.db.char["mounts100"][i])
				if ( name == prefMount[1] ) then
					prefMount[2] = tonumber(bag)
					prefMount[3] = tonumber(slot)
					return
				end
			end
		end
		for i in ipairs(self.db.char["mounts60"]) do
			local name, bag, slot = strsplit("&", self.db.char["mounts60"][i])
			if ( name == prefMount[1] ) then
				prefMount[2] = tonumber(bag)
				prefMount[3] = tonumber(slot)
				return
			end
		end
	end
	local tbl
	if (( #self.db.char["mounts100"] >= 1 ) and ( ridingSkill >= 150 )) then
		tbl = "mounts100"
	elseif ( #self.db.char["mounts60"] >= 1 ) then
		tbl = "mounts60"
	else
		return
	end
	local name, bag, slot = strsplit("&", self.db.char[tbl][1])
	self.db.char.prefMount = { name, tonumber(bag), tonumber(slot) }
end

function SacredBuff:UpdatePrefAQMount()
	if ( self.db.char.prefAQMount ) then
		for i in ipairs(self.db.char["mountsaq100"]) do
			local name, bag, slot = strsplit("&", self.db.char["mountsaq100"][i])
			if ( name == self.db.char.prefAQMount[1] ) then
				self.db.char.prefAQMount[2] = tonumber(bag)
				self.db.char.prefAQMount[3] = tonumber(slot)
				return
			end
		end	
	end
	if ( #self.db.char["mountsaq100"] >= 1 ) then
		local name, bag, slot = strsplit("&", self.db.char["mountsaq100"][1])
		self.db.char.prefAQMount = { name, tonumber(bag), tonumber(slot) }
	else
		self.db.char.prefAQMount = nil
	end
end

function SacredBuff:IterateMounts(mountType, quality)
	local tableLoc = self.db.char[mountType..quality]
	for i in ipairs(tableLoc) do
		tableLoc[i] = nil
	end
	for bag, slot, level in PT:BagIter(mountType) do
		if ( level == quality ) then
			local itemName = GetItemInfo( GetContainerItemLink(bag, slot) )
			table.insert( tableLoc, strjoin("&", itemName, bag, slot, quality) )
		end
	end
end


--[[---------------------------------------------------------------------------------
 OTHER MISCELLANEOUS YET REALLY NEAT-O FUNCTIONS
------------------------------------------------------------------------------------]]
function SacredBuff:UpdateCandleCount()
	self.db.char.holyCount = GetItemCount(17028)
	self.db.char.sacredCount = GetItemCount(17029)
	self.frame.main:SetText(self.db.char.sacredCount)
end

function SacredBuff:FindSpell(spell)
	local tbl
	local i = 1
	while true do
		local spellName, spellRank = GetSpellName(i, BOOKTYPE_SPELL)
		if ( not spellName ) then
			break
		end
		if ( spellName == spell ) then
			if ( spellRank ) then
				local _, _, r = string.find(spellRank, RANK.."%s(%d+)")
				spellRank = r or 0
			end
			G:Erase()
			G:SetSpell(i, BOOKTYPE_SPELL)
			local _, _, manaCost = G:Find("(%d+)%s"..MANA, 2, 2, false, true)
			if ( not manaCost ) then
				manaCost = "0"
			end
			if ( not tbl ) then
				tbl = { i, spellRank, manaCost, spellName }
			else
				if ( tbl[2] < spellRank ) then
					tbl[1] = i
					tbl[2] = spellRank
					tbl[3] = manaCost
				end
			end
		end
		i = i + 1
	end
	return tbl
end